import numpy as np
import glob
import netCDF4
import os, sys
from datetime import datetime, timedelta
import inspect

current_dir = os.path.dirname(os.path.abspath(inspect.getfile(inspect.currentframe())))
sys.path.insert(0, os.path.dirname(current_dir))

from lightningcast import utils

bird = "goes_west"

globstr = f"/ships22/grain/probsevere/LC/T2O_validation/{bird}_conus/all/month??/spatial_counts*.nc"
# globstr = f'/ships19/grain/jcintineo/GLM/{bird}/FD/2020*-*/spatial_counts.nc'
files = np.sort(glob.glob(globstr))
outdir = f"/ships22/grain/probsevere/LC/T2O_validation/{bird}_conus/all/" 
os.makedirs(outdir, exist_ok=True)

all_samples = 0
for ii, f in enumerate(files):
    print(f)
    nc = netCDF4.Dataset(f, "r")
    hits = nc["hits"][:]
    misses = nc["misses"][:]
    FAs = nc["FAs"][:]
    thresholds = nc["thresholds"][:]
    nsamples = int(nc.sample_shape.split("(")[1].split(",")[0])
    nc.close()
    all_samples += nsamples

    if ii == 0:
        all_hits = hits + 0.0
        all_misses = misses + 0.0
        all_FAs = FAs + 0.0
    else:
        all_hits += hits
        all_misses += misses
        all_FAs += FAs

    shape = all_hits.shape

datasets = {}
dims = {"t": len(thresholds), "y": shape[1], "x": shape[2]}
datasets["hits"] = {"data": all_hits, "dims": ("t", "y", "x"), "atts": {}}
datasets["misses"] = {"data": all_misses, "dims": ("t", "y", "x"), "atts": {}}
datasets["FAs"] = {"data": all_FAs, "dims": ("t", "y", "x"), "atts": {}}
datasets["thresholds"] = {"data": thresholds, "dims": ("t"), "atts": {}}
global_atts = {"samplelist": "various", "files": globstr}
global_atts["sample_shape"] = f"({all_samples},{shape[1]},{shape[2]})"
global_atts["created"] = datetime.utcnow().strftime("%Y%m%d-%H%M UTC")
utils.write_netcdf(
    f"{outdir}/spatial_counts.nc", datasets, dims, atts=global_atts, gzip=False
)
