import glob
import os
import argparse
import subprocess
import time
from datetime import datetime
pltg_data = os.environ['PLTG_DATA']

parser = argparse.ArgumentParser(
    description="Checks the status of real-time products (e.g., netCDF, json) and "
    + "sends an email if the latency is too long."
)
parser.add_argument(
    "--goes_east",
    help="Check goes_east sectors (RadM1, RadM2, RadC, RadF).",
    action="store_true",
)
parser.add_argument(
    "--goes_west",
    help="Check goes_west sectors (RadM1, RadM2, RadC, RadF).",
    action="store_true",
)
parser.add_argument(
    "--himawari",
    help="Check himawari sectors (FLDK).",
    action="store_true",
)
parser.add_argument(
    "--email",
    help="Send problem messages to this email address. Default=john.cintineo@noaa.gov.",
    type=str,
    default='john.cintineo@noaa.gov'
)

args = parser.parse_args()

subject = 'LightningCast processing problem'

satellites = []
if args.goes_east: satellites.append('goes_east')
if args.goes_west: satellites.append('goes_west')
if args.himawari: satellites.append('ahi_japan')

while True:

    latest_date = sorted(glob.glob(f'{pltg_data}/products/20*'))[-1]
    body = ""
    nowsecs = int(time.time())
    time_thresh_FD = 40 # minutes
    old_time_FD = nowsecs - 60 * time_thresh_FD
    time_thresh_RadC = 15 # minutes
    old_time_RadC = nowsecs - 60 * time_thresh_RadC
    time_thresh_RadM = 5 # minutes
    old_time_RadM = nowsecs - 60 * time_thresh_RadM

    for sat in satellites:

        sectors = sorted(glob.glob(f'{latest_date}/{sat}/*'))

        for sector in sectors:
            products = glob.glob(f'{sector}/*')

            if 'RadM1' in sector or 'RadM2' in sector:
                old_time = old_time_RadM
                time_thresh = time_thresh_RadM
            elif 'RadC' in sector:
                old_time = old_time_RadC
                time_thresh = time_thresh_RadC
            else:
                old_time = old_time_FD
                time_thresh = time_thresh_FD

            for product in products:
                if 'json' in product:
                    json_prods = glob.glob(f'{product}/*')
                    for json_prod in json_prods:
                        latest_file = sorted(glob.glob(f'{json_prod}/*'))[-1]
                else:
                    latest_file = sorted(glob.glob(f'{product}/*'))[-1]

                #mtime_dt = datetime.fromtimestamp(os.path.getmtime(latest_file)) 
                mtime = os.path.getmtime(latest_file)
      
                if mtime < old_time:
                    body += f"{latest_file} is more than {time_thresh} minutes old.\n"

        body += '\n' 


    if len(body) > 20:
       
        of = open(f'{pltg_data}/err_mail.txt','w')
        of.write(body)
        of.close()
        
        #Below is the same as:
        # printf $BODY | /bin/email -s $SUBJECT $ADDRESS      
 
        cmd1 = subprocess.run(['printf', f'{body}'], capture_output=True).stdout
        decoded_cmd1 = cmd1.decode("utf-8")
        cmd2 = subprocess.run(['/bin/mail','-s',f'{subject}',f'{args.email}'], input=decoded_cmd1, text=True)
      
        print('sent email')

    time.sleep(5*60)

