import os, sys
import numpy as np
from lightningcast import solar_angles
import netCDF4
import time
import pyproj
import glob
from datetime import datetime
import matplotlib.pyplot as plt
from lightningcast import utils
import collections

t0 = time.time()
# files = ['/apollo/grain/saved_probsevere_data/lightning/20190701/GLM/goes_east/FED_accum_60min/20190701-235600.netcdf']

files = np.sort(
    glob.glob(
        "/apollo/grain/saved_probsevere_data/lightning/20191*/GLM/goes_east/FED_accum_60min/*.netcdf"
    )
)

# print(len(files))
# print(files[0])
# print(files[-1])
# sys.exit()

for f in files:
    print(f)
    data = {
        "FED_accum_60min": {"atts": ["ALL"]},
        "x": {"atts": ["ALL"]},
        "y": {"atts": ["ALL"]},
        "goes_imager_projection": {"atts": ["ALL"]},
    }
    gatts = {
        "subsatellite_longitude": -1,
        "platform_ID": -1,
        "orbital_slot": -1,
        "spatial_resolution": -1,
        "time_coverage_start": -1,
        "time_coverage_end": -1,
        "aggregation": -1,
    }
    s = utils.read_netcdf(f, datasets=data, global_atts=gatts)

    try:
        projection
    except (NameError, UnboundLocalError):
        projection = pyproj.Proj(
            proj="geos",
            h=data["goes_imager_projection"]["atts"]["perspective_point_height"],
            lon_0=data["goes_imager_projection"]["atts"][
                "longitude_of_projection_origin"
            ],
            sweep=data["goes_imager_projection"]["atts"]["sweep_angle_axis"],
        )

    # this section is to fix X and Y, which were not originally written as uint16
    sf = data["x"]["atts"]["scale_factor"]
    ao = data["x"]["atts"]["add_offset"]
    x = (data["x"]["data"] - ao) / sf  # unscale to get correct values
    #  X = np.copy(x)
    x = (x - ao) / sf  # reverse-scale so we can save as uint16
    data["x"]["data"] = np.round(x).astype(np.uint16)

    sf = data["y"]["atts"]["scale_factor"]
    ao = data["y"]["atts"]["add_offset"]
    y = (data["y"]["data"] - ao) / sf  # unscale to get correct values
    #  Y = np.copy(y)
    y = (y - ao) / sf  # reverse-scale so we can save as uint16
    data["y"]["data"] = np.round(y).astype(np.uint16)

    # calc solzen
    #  x = X*data['goes_imager_projection']['atts']['perspective_point_height']
    #  y = Y*data['goes_imager_projection']['atts']['perspective_point_height']
    #  irxx,iryy = np.meshgrid(x,y)
    #  irlons, irlats = projection(irxx, iryy, inverse=True)
    #  dt = datetime.strptime(os.path.basename(f),'%Y%m%d-%H%M%S.netcdf')
    #  solzen, solaz = solar_angles.solar_angles(dt=dt,lat=irlats,lon=irlons)
    #  data['solzen'] = {'data':solzen.astype('f4'),'atts':{'units':'degrees','desc':'solar-zenith angle'},'dims':('y','x')}

    dims = collections.OrderedDict()
    dims["x"] = len(x)
    dims["y"] = len(y)
    dims["g"] = 1

    data["x"]["dims"] = "x"
    data["y"]["dims"] = "y"
    data["goes_imager_projection"]["dims"] = "g"
    data["FED_accum_60min"]["dims"] = ("y", "x")

    outfile = f
    status = utils.write_netcdf(
        outfile, data, dims, gatts, wait=False, gzip=False, nc4=True
    )


print(time.time() - t0)
sys.exit()
plt.imshow(solzen)
plt.colorbar()
plt.show()
plt.imshow(irlats)
plt.show()
plt.imshow(irlons)
plt.show()
