import netCDF4
import json
import numpy as np
import satpy
import glob
from lightningcast import utils
from datetime import datetime
from lightningcast.sat_zen_angle import sat_zen_angle

# For AKCAN
#outprefix = "AKCAN"
#startX, endX, startY, endY = (2020, 3500, 50, 500)

# For USSAMOA
outprefix='USSAMOA'
startX, endX, startY, endY = (-180, -155, -22, -4)

# For GUAM
# outprefix='GUAM'
# startX, endX, startY, endY = (140, 150, 11, 18)

dt = datetime(2023, 9, 5, 17, 40)
all_feats = {
    "validTime": dt.strftime("%Y%m%d_%H%M%S UTC"),
    "type": "FeatureCollection",
    "features": [],
}

if outprefix == "GUAM" or outprefix == "USSAMOA":
    latlons = [
        (startX, startY),
        (startX, endY),
        (endX, endY),
        (endX, startY),
        (startX, startY),
    ]

    all_feats["features"].append(
        {
            "type": "Feature",
            "geometry": {"type": "Polygon", "coordinates": [latlons]},
            "properties": {
                "OPACITY": "0",
                "BOPACITY": "100",
                "COLOR": "255 0 0",
                "BCOLOR": "255 0 0",
                "WIDTH": "3",
            },
        }
    )


if outprefix == "AKCAN":
    abi_filenames = glob.glob(
        dt.strftime(
            f"/arcdata/goes/grb/goes18/%Y/%Y_%m_%d_%j/abi/L1b/RadF/OR*C13*s%Y%j%H%M*nc"
        )
    )
    orig_scn = satpy.Scene(reader="abi_l1b", filenames=abi_filenames)
    orig_scn.load(["C13"])
    projx = orig_scn["C13"].x.compute().data
    projy = orig_scn["C13"].y.compute().data

    xmin = projx[startX]
    ymin = projy[startY]
    xmax = projx[endX]
    ymax = projy[endY]
    xy_bbox = [xmin, ymin, xmax, ymax]
    scn = orig_scn.crop(xy_bbox=xy_bbox)

    irlons, irlats = scn["C13"].attrs["area"].get_lonlats()

    #  lon_0 = scn['C13'].attrs['orbital_parameters']['projection_longitude']
    #  satzen = sat_zen_angle(irlats,irlons,satlon=lon_0)

    nlats, nlons = irlons.shape

    # Start in the southwest corner and go north
    startpt = (irlons[-1, 0] % 360, irlats[-1, 0])
    latlons = []

    for ii in range(nlats - 1, -1, -1):
        if irlons[ii, 0] != np.inf:
            latlons.append((irlons[ii, 0] % 360, irlats[ii, 0]))

    # Now head east
    for jj in range(nlons):
        if irlons[0, jj] != np.inf:
            latlons.append((irlons[0, jj] % 360, irlats[0, jj]))

    # Now head south
    for ii in range(nlats):
        if irlats[ii, -1] != np.inf:
            latlons.append((irlons[ii, -1] % 360, irlats[ii, -1]))

    # add starting point again. Assuming straight line across and no space looks
    # latlons.append(startpt)
    latlons.append(startpt)

    # Now make polygon with lons -360 to 0. This is a hack to get it to work across the dateline.
    # We need TWO polygons: one 0 to +360 and one -360 to 0.

    latlonsNEG = []
    for ll in latlons:
        latlonsNEG.append(((ll[0] - 360) % (-360), ll[1]))

    for ll in [latlons, latlonsNEG]:
        all_feats["features"].append(
            {
                "type": "Feature",
                "geometry": {"type": "Polygon", "coordinates": [ll]},
                "properties": {
                    "OPACITY": "0",
                    "BOPACITY": "100",
                    "COLOR": "255 0 0",
                    "BCOLOR": "255 0 0",
                    "WIDTH": "3",
                    "INFO": "LightningCast Alaska/western Canada sector boundary",
                },
            }
        )


json_obj = json.dumps(all_feats, indent=2)

with open(
    f'{outprefix}-boundary_{dt.strftime("%Y%m%d_%H%M%S")}.force.json', "w"
) as outfile:
    outfile.write(json_obj)
