import numpy as np
import pandas as pd

# import xarray as xr
from glob import glob
import os, sys
from datetime import datetime, timedelta
import time
from multiprocessing import Pool
from netCDF4 import Dataset
from lightningcast import utils
from subprocess import call, Popen, PIPE

sys.path.insert(0, "..")

t0 = time.time()


def get_data(ncs):

    epoch0 = datetime(1970, 1, 1, 0, 0)

    recs = {
        "CH02": [],
        "CH05": [],
        "CH13": [],
        "FED": [],
        "maxFED_t0": [],
        "maxFED_t5": [],
        "maxFED_t10": [],
        "maxFED_t15": [],
        "maxFED_t20": [],
        "maxFED_t25": [],
        "maxFED_t30": [],
        "timestamp": [],
        "lat": [],
        "lon": [],
        "SST": [],
        "mlcape": [],
        "u925": [],
        "v925": [],
        "u850": [],
        "v850": [],
        "u700": [],
        "v700": [],
        "u500": [],
        "v500": [],
        "ltg5to30": [],
        "ABIx": [],
        "ABIy": [],
        "ABIx10": [],
        "ABIy10": [],
        "ABIx05": [],
        "ABIy05": [],
    }
    variables = ["CH02", "CH05", "CH13", "FED"]
    atts = [
        "lat",
        "lon",
        "maxFED_t0",
        "maxFED_t5",
        "maxFED_t10",
        "maxFED_t15",
        "maxFED_t20",
        "maxFED_t25",
        "maxFED_t30",
        "SST",
        "mlcape",
        "u925",
        "v925",
        "u850",
        "v850",
        "u700",
        "v700",
        "u500",
        "v500",
    ]
    intatts = ["ABIx", "ABIy", "ABIx10", "ABIy10", "ABIx05", "ABIy05"]

    for s in ncs:
        print(s)
        # goes16_conus_Y112_X2182_20180611-001719.netcdf
        # find previous seq_len times
        # basefile = os.path.basename(s)
        # parts = basefile.split('_')
        # dt = datetime.strptime(parts[-1][0:15],'%Y%m%d-%H%M%S')
        # prev_dts = [dt - timedelta(minutes=sl*5) for sl in range(seq_len)]
        # prev_dts.reverse()

        if s[-3:] == ".gz":
            _ = call(["gunzip", "-f", s])
            unzipped_file = s[0:-3]
            the_file = unzipped_file
            gzip = True
        else:
            gzip = False
            the_file = s

        try:
            nc = Dataset(the_file, "r")
        except OSError as err:
            f = open("bad_files.txt", "w+")
            f.write(s + "\n")
            f.close()
            continue

        maxFED_t0 = nc.maxFED_t0
        maxFED_t5 = nc.maxFED_t5
        maxFED_t10 = nc.maxFED_t10
        maxFED_t15 = nc.maxFED_t15
        maxFED_t20 = nc.maxFED_t20
        maxFED_t25 = nc.maxFED_t25
        maxFED_t30 = nc.maxFED_t30
        ltg5to30 = (
            1
            if (
                np.max([maxFED_t10, maxFED_t15, maxFED_t20, maxFED_t25, maxFED_t30])
                >= 1
            )
            else 0
        )
        recs["ltg5to30"].append(np.int8(ltg5to30))

        dt = int(
            (
                datetime.strptime(nc.ABI_time[0:15], "%Y%m%d-%H%M%S") - epoch0
            ).total_seconds()
        )
        recs["timestamp"].append(dt)

        for att in atts:
            recs[att].append(np.float32(nc.getncattr(att)))
        for att in intatts:
            recs[att].append(np.int16(nc.getncattr(att)))

        for var in variables:
            recs[var].append(nc.variables[var][:])

        nc.close()

        if gzip:
            _ = Popen(
                ["gzip", "-f", unzipped_file]
            )  # ==will run in background since it doesn't need output from command.

    return recs


##################################################################################################################

if __name__ == "__main__":

    nthreads = 12
    chunksize = 300  # number of images/samples to chunk or read at once
    # seq_len = 7

    pool = Pool(nthreads)

    ncdir = "/data/jcintineo/AI/OCONUS/netcdf/"

    allncs = np.sort(glob(os.path.join(ncdir, "20180726", "*.netcdf*")))
    interval = int(np.ceil(len(allncs) / nthreads))

    # get some atts
    global_atts = {"GLM_truth_buffer": 0, "predict_leadtime": 0, "spatial_stride": 0}
    if allncs[0][-3:] == ".gz":
        _ = call(["gunzip", "-f", allncs[0]])
        unzipped_file = allncs[0][0:-3]
        the_file = unzipped_file
        gzip = True
    else:
        gzip = False
        the_file = allncs[0]
    nc = Dataset(the_file, "r")
    for gatt in global_atts:
        global_atts[gatt] = nc.getncattr(gatt)
    nc.close()
    if gzip:
        _ = Popen(["gzip", "-f", unzipped_file])

    # divvy up the netcdfs
    list_of_lists = []
    for n in range(nthreads):
        list_of_lists.append(allncs[n * interval : n * interval + interval])
        # print(n*interval,n*interval+interval+1)

    import itertools

    results = pool.starmap(get_data, zip(list_of_lists))  # ,itertools.repeat(seq_len)))
    pool.close()
    pool.join()
    # results is a list of dicts

    # concatenate lists in results
    # results = list(itertools.chain.from_iterable(results))

    fields = {
        "lat": [],
        "lon": [],
        "timestamp": [],
        "CH02": [],
        "CH05": [],
        "CH13": [],
        "FED": [],
        "maxFED_t0": [],
        "maxFED_t5": [],
        "maxFED_t10": [],
        "maxFED_t15": [],
        "maxFED_t20": [],
        "maxFED_t25": [],
        "maxFED_t30": [],
        "SST": [],
        "mlcape": [],
        "u925": [],
        "v925": [],
        "u850": [],
        "v850": [],
        "u700": [],
        "v700": [],
        "u500": [],
        "v500": [],
        "ltg5to30": [],
        "ABIx": [],
        "ABIy": [],
        "ABIx10": [],
        "ABIy10": [],
        "ABIx05": [],
        "ABIy05": [],
    }

    datasets = {}
    for f in fields:
        for ii in range(
            len(results)
        ):  # concatenate list of results (dicts) and then stck them into an array
            fields[f] += results[ii][f]
        fields[f] = np.stack(fields[f])
        datasets[f] = {"atts": []}
        datasets[f]["data"] = fields[f]

    dims = {}
    dims["x"], dims["y"] = np.shape(datasets["CH13"]["data"][0])
    dims["x05"], dims["y05"] = np.shape(datasets["CH02"]["data"][0])
    dims["x10"], dims["y10"] = np.shape(datasets["CH05"]["data"][0])
    dims["t"] = len(datasets["timestamp"]["data"])

    for v in datasets:
        # datasets[v]['atts'] = vars_to_write[v]['atts'] #this sets the metadata for v
        if v == "CH02":
            currdim = ("t", "x05", "y05")
            # datasets[v]['chunksizes'] = {'t':1024,'x05':dims['x05'],'y05':dims['y05']}
            datasets[v]["chunksizes"] = [chunksize, dims["x05"], dims["y05"]]
        elif v == "CH05":
            currdim = ("t", "x10", "y10")
            # datasets[v]['chunksizes'] = {'t':1024,'x10':dims['x10'],'y10':dims['y10']}
            datasets[v]["chunksizes"] = [chunksize, dims["x10"], dims["y10"]]
        elif v in [
            "CH06",
            "CH07",
            "CH08",
            "CH10",
            "CH13",
            "CH14",
            "FED",
            "FCD",
            "TOE",
            "AFA",
        ]:
            currdim = ("t", "x", "y")
            # datasets[v]['chunksizes'] = {'t':1024,'x':dims['x'],'y':dims['y']}
            datasets[v]["chunksizes"] = [chunksize, dims["x"], dims["y"]]
        else:
            currdim = ("t",)
            # datasets[v]['chunksizes'] = {'t':1024}
            datasets[v]["chunksizes"] = [chunksize]
        datasets[v]["dims"] = currdim  # this sets the dimensions for v

    output_file = os.path.join("/data", "jcintineo", "AI", "OCONUS", "20180726.nc")
    utils.write_netcdf(output_file, datasets, dims, atts=global_atts)

    print(time.time() - t0)
