import numpy as np
import pandas as pd
import xarray as xr
from glob import glob
import os, sys
from datetime import datetime, timedelta
import time
from multiprocessing import Pool
from netCDF4 import Dataset

sys.path.insert(0, "..")

t0 = time.time()


def get_data(ncs, seq_len):

    records = []
    for s in ncs:
        print(s)
        # goes16_conus_Y112_X2182_20180611-001719.netcdf
        # find previous seq_len times
        basefile = os.path.basename(s)
        dirname = os.path.dirname(s)
        dirbase = os.path.dirname(dirname)  # remove yyyymmdd
        parts = basefile.split("_")
        dt = datetime.strptime(parts[-1][0:15], "%Y%m%d-%H%M%S")
        prev_dts = [dt - timedelta(minutes=sl * 5) for sl in range(seq_len)]
        prev_dts.reverse()

        ABI_time, ch13 = [[] for _ in range(2)]

        cont = False
        for pp in range(len(prev_dts)):
            try:
                pnc = glob(
                    prev_dts[pp].strftime(
                        os.path.join(
                            dirbase,
                            "%Y%m%d",
                            ("_").join(
                                [
                                    parts[0],
                                    parts[1],
                                    parts[2],
                                    parts[3],
                                    "%Y%m%d-%H%M*.netcdf",
                                ]
                            ),
                        )
                    )
                )[0]
            except IndexError:
                cont = True
                break
            else:
                nc = Dataset(pnc, "r")
                if pp == 0:
                    lat = nc.lat
                    lon = nc.lon
                    maxFED_t0 = nc.maxFED_t0
                    maxFED_t5 = nc.maxFED_t5
                    maxFED_t10 = nc.maxFED_t10
                    maxFED_t15 = nc.maxFED_t15
                    maxFED_t20 = nc.maxFED_t20
                    maxFED_t25 = nc.maxFED_t25
                    maxFED_t30 = nc.maxFED_t30
                    ltg5to30 = (
                        1
                        if (
                            np.max(
                                [
                                    maxFED_t10,
                                    maxFED_t15,
                                    maxFED_t20,
                                    maxFED_t25,
                                    maxFED_t30,
                                ]
                            )
                            >= 1
                        )
                        else 0
                    )
                    mlcape = nc.mlcape
                    sst = nc.SST
                    u925 = nc.u925
                    v925 = nc.v925
                    u850 = nc.u850
                    v850 = nc.v850
                    u700 = nc.u700
                    v700 = nc.v700
                    u500 = nc.u500
                    v500 = nc.v500

                ABI_time.append(datetime.strptime(nc.ABI_time[0:15], "%Y%m%d-%H%M%S"))
                # ch02.append((nc.CH02.data).astype(np.float16))
                # ch05.append((nc.CH05.data).astype(np.float16))
                ch13.append((nc.variables["CH13"][:]).astype(np.float16))
                # fed.append((nc.FED.data).astype(np.float16)

        if cont:
            continue

        records.append(
            [
                lat,
                lon,
                ABI_time,
                ch13,
                maxFED_t0,
                maxFED_t5,
                maxFED_t10,
                maxFED_t15,
                maxFED_t20,
                maxFED_t25,
                maxFED_t30,
                sst,
                mlcape,
                u925,
                v925,
                u850,
                v850,
                u700,
                v700,
                u500,
                v500,
                ltg5to30,
            ]
        )

        nc.close()

    return records


##################################################################################################################

if __name__ == "__main__":

    nthreads = 12

    seq_len = 7

    pool = Pool(nthreads)

    ncdir = "/data/jcintineo/AI/OCONUS/netcdf/"

    allncs = np.sort(glob(os.path.join(ncdir, "20180[5,6,7]*", "*.netcdf*")))
    interval = int(np.ceil(len(allncs) / nthreads))

    list_of_lists = []

    for n in range(nthreads):
        list_of_lists.append(allncs[n * interval : n * interval + interval + 1])
        # print(n*interval,n*interval+interval+1)

    import itertools

    results = pool.starmap(get_data, zip(list_of_lists, itertools.repeat(seq_len)))
    pool.close()
    pool.join()

    # concatenate lists in results
    results = list(itertools.chain.from_iterable(results))

    df = pd.DataFrame.from_records(results)

    fields = [
        "lat",
        "lon",
        "ABI_time",
        "ch13",
        "maxFED_t0",
        "maxFED_t5",
        "maxFED_t10",
        "maxFED_t15",
        "maxFED_t20",
        "maxFED_t25",
        "maxFED_t30",
        "sst",
        "mlcape",
        "u925",
        "v925",
        "u850",
        "v850",
        "u700",
        "v700",
        "u500",
        "v500",
        "ltg5to30",
    ]

    df.columns = fields

    # df.sort_values(by='Date')

    df.to_pickle("ch13_mayjunjul_seq.pkl")
    print(time.time() - t0)
